@@ -22,6 +22,7 @@ import time
 import pandas as pd
 import matplotlib.patches as mplpatches
 import pickle
+import bisect
 
 
 
@@ -29,6 +30,28 @@ import pickle
 
 
 def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, sample, percentage, signature, panel1, panel3, panel5, fig, chrom=None):
+	'''
+	Plots the SBS96 catalogs.
+
+	Parameters:
+					 matrix_path	-> 	path to SBS96 matrix (string)
+		   matrix_path_clustered	-> 	path to SBS96 matrix of clustered mutations (string)
+		matrix_path_nonClustered	->	path to SBS96 matrix of non-clustered mutations (string)
+						  sample	->	name of current sample (string)
+					  percentage	->	optional parameter to normalize plots (boolean; default=False)
+					   signature	-> 	optional parameter to generate plots using floats (boolean; default=False)
+						  panel1	->	axis information for the SBS96 plot of all mutations
+						  panel3	->	axis information for the SBS96 plot of clustered mutations
+						  panel5	->	axis information for the SBS96 plot of non-clustered mutations
+						     fig	->	the matplotlib object for the current figure
+						   chrom	-> 	DO NOT USE
+
+	Returns:
+		None
+
+	Outputs:
+		Plots for all of the SBS96 axes
+	'''
 	if 'roman' in matplotlib.font_manager.weight_dict:
 		del matplotlib.font_manager.weight_dict['roman']
 		matplotlib.font_manager._rebuild()
@@ -363,6 +386,27 @@ def plot96_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, s
 	plt.text(.008, .55, 'Mutation Counts', rotation='vertical', fontsize=20, fontweight='bold', fontname="Times New Roman", transform=plt.gcf().transFigure)
 
 def plotINDEL_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered, sample, percentage, signature, panel1, panel3, panel5, fig):
+	'''
+	Plots the ID83 catalogs.
+
+	Parameters:
+					 matrix_path	-> 	path to SBS96 matrix (string)
+		   matrix_path_clustered	-> 	path to SBS96 matrix of clustered mutations (string)
+		matrix_path_nonClustered	->	path to SBS96 matrix of non-clustered mutations (string)
+						  sample	->	name of current sample (string)
+					  percentage	->	optional parameter to normalize plots (boolean; default=False)
+					   signature	-> 	optional parameter to generate plots using floats (boolean; default=False)
+						  panel1	->	axis information for the SBS96 plot of all mutations
+						  panel3	->	axis information for the SBS96 plot of clustered mutations
+						  panel5	->	axis information for the SBS96 plot of non-clustered mutations
+						     fig	->	the matplotlib object for the current figure
+
+	Returns:
+		None
+
+	Outputs:
+		Plots for all of the ID83 axes
+	'''
 	if 'roman' in matplotlib.font_manager.weight_dict:
 		del matplotlib.font_manager.weight_dict['roman']
 		matplotlib.font_manager._rebuild()
@@ -933,16 +977,37 @@ def plotINDEL_same (matrix_path, matrix_path_clustered, matrix_path_nonClustered
 
 
 
-def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 10000000):
+def rainfall (chrom_based_IMD, project, project_path, chrom_path, correction=True, windowSize = 10000000):
+	'''
+	Generates rainfall plots when subClassify is True.
+
+	Parameters:
+			chrom_based_IMD	->	optional parameter for generating plots w.r.t chromosome-based IMDs (boolea; default=False)
+					project	->	user provided project name (string)
+			   project_path	-> 	the directory for the given project (string)
+				 chrom_path	->	path to the reference genome chromosome files used for SigProfilerMatrixGenerator (string)
+				 correction	-> 	optional parameter that corrects for mutational density (boolean; default=True)
+				 windowSize	->	the window size used to calculate the mutation densities across the genome (integer; default=None)
+
+	Returns:
+		None
+
+	Outputs:
+		Rainfall plots for each sample.
+	'''
 	aggregated = False
 	chrom_based = False
 
 	path_suffix = ''
 	if chrom_based_IMD:
 		path_suffix = '_chrom'
+	path_suffix2 = ''
+	if correction:
+		path_suffix2 += "_corrected"
+
 	projectPath_parent2 = project_path
 	projectPath_parent = project_path
-	projectPath = project_path + "output/vcf_files/ESCC_clustered/subclasses" + path_suffix + "/"
+	projectPath = project_path + "output/vcf_files" + path_suffix2 + "/" + project + "_clustered/subclasses" + path_suffix + "/"
 	outputPath = projectPath_parent + "output/plots/"
 
 	if not os.path.exists(outputPath):
@@ -955,18 +1020,24 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 	if chrom_based_IMD:
 		plot_suffix += "_chrom"
 
-	classes = ["Class IA", "Class IB", "Class IC", "Class II", "Class III", "Non-clust"]
-	mutationsPath = [projectPath + "class1a/ESCC_clustered_class1a.txt", projectPath + "class1b/ESCC_clustered_class1b.txt", projectPath + "class1c/ESCC_clustered_class1c.txt",
-					projectPath + "class2/ESCC_clustered_class2.txt", projectPath + "class3/ESCC_clustered_class3.txt", projectPath_parent + "output/vcf_files/ESCC_nonClustered/SNV/ESCC_nonClustered.txt"]
+	if correction:
+		plot_suffix += "_corrected"
+	classes = ["Class IA", "Class II", "Class IB", "Class IC",  "Class III", "Non-clust"]
+	mutationsPath = [projectPath + "class1a/" + project + "_clustered_class1a.txt", projectPath + "class2/" + project + "_clustered_class2.txt", projectPath + "class1b/" + project + "_clustered_class1b.txt", projectPath + "class1c/" + project + "_clustered_class1c.txt",
+					 projectPath + "class3/" + project + "_clustered_class3.txt", projectPath_parent + "output/vcf_files" + path_suffix2 + "/" + project + "_nonClustered/SNV/" + project + "_nonClustered.txt"]
+
 
 	with open(projectPath_parent2 + "output/simulations/data/imds" + path_suffix + ".pickle", "rb") as handle:
 		imdsData = pickle.load(handle) 
 
 	with open(projectPath_parent2 + "output/simulations/data/imds.pickle", "rb") as handle:
 		imdsDataSample = pickle.load(handle) 
+	if correction:
+		with open(projectPath_parent2 + "output/simulations/data/imds_corrected.pickle", "rb") as handle:
+			imdsDataSample_corrected = pickle.load(handle) 
 
 
-	headerFields = ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "vaf","class", "failedReason"]
+	headerFields = ["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMD", "vaf","class", "failedReason"]
 
 	centromeres = {'GRCh38':{'1': [122026460,125184587],'10': [39686683,41593521],'11':[51078349,54425074],'12':[34769408,37185252],'13': [16000001,18051248],
 				'14': [16000001,18173523],'15': [17000001,19725254],'16': [36311159,38280682],'17': [22813680,26885980],'18': [15460900,20861206],
@@ -1000,14 +1071,15 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 	samples = {}
 	allMutations = {}
 	if classes[0] == "Class III":
-		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields, header=None)
+		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields, header=0)
 	elif classes[0] == "Non-clust":
-		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields[:-3], header=None, skiprows=[0])
+		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields[:-3], header=0)#, skiprows=[0])
 	elif classes[0] == "Simulation":
 		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=["project", "ID","sim","genome","chr","start","end","strand","placeHolder","mutType", "ref", "alt1","alt", "placeHolder2","placeHolder3","samples", "seq"], header=None, skiprows=[0])
 	else:
-		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields[:-1], header=None)
+		mutations = pd.read_csv(mutationsPath[0], sep="\t", names=headerFields[:-1], header=0)
 	genome = mutations.head(1).loc[0,'genome']
+	mutations['chr'] = mutations['chr'].astype(str)
 	mutations = mutations.set_index(['samples', 'chr'])
 	mutations = mutations.sort_index()
 	allMutations[classes[0]] = mutations
@@ -1016,11 +1088,12 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 	if len(mutationsPath) > 1:
 		for i in range(1, len(mutationsPath), 1):
 			if classes[i] == "Class III":
-				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields, header=None)
+				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields, header=0, skiprows=[0],)
 			elif classes[i] == "Non-clust":
-				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields[:-3], header=None, skiprows=[0])
+				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=["project", "samples","ID","genome","mutType","chr","start","end", "ref", "alt", "mutClass", "IMD"], header=0, skiprows=[0], engine='python')
 			else:
-				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields[:-1], header=None)
+				newMutations = pd.read_csv(mutationsPath[i], sep="\t", names=headerFields[:-1], header=0)
+			newMutations['chr'] = newMutations['chr'].astype(str)
 			newMutations = newMutations.set_index(['samples', 'chr'])
 			newMutations = newMutations.sort_index()
 			allMutations[classes[i]] = newMutations
@@ -1031,13 +1104,37 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 	# chrom_path = "/anaconda3/lib/python3.6/site-packages/SigProfilerMatrixGenerator/references/chromosomes/tsb/" + genome + "/"
 	chroms = [x.split(".")[0] for x in os.listdir(chrom_path) if x != ".DS_Store" and x[0]!= "G" and x[0] != "M" and x != "BED_" + genome + "_proportions.txt"]
 	chroms = sorted(chroms, key = lambda x: ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22','X','Y'].index(x[0:]))
-	pp = PdfPages(outputPath + "rainfallPlots_clustered_" + plot_suffix + ".pdf")
-	colors = ['red', 'black', 'green', 'orange', 'blue', 'grey']
+	pp = PdfPages(outputPath + "rainfallPlots_clustered_" + plot_suffix +".pdf")
+	colors = ['red',  'orange','black', 'green',  'blue', 'grey']
 	colors = {'Class IA':'red', "Class IB":'black', "Class IC":'green', "Class II":'orange', "Class III":'blue', "Non-clust":'grey', "Simulation":'grey'}
 
+	if genome not in chromLengths:
+		chromLengths[genome] = {}
+		for chrom in chroms:
+			with open(chrom_path + chrom + ".txt", "rb") as f:
+				chromLengths[genome][chrom] = len(f.read())
+
+	if correction:
+		chromLengths2 = {}
+		chroms = [x.split(".")[0] for x in os.listdir(chrom_path) if x != ".DS_Store" and x[0]!= "G" and x[0] != "M" and x != "BED_" + genome + "_proportions.txt"]
+		chroms = sorted(chroms, key = lambda x: ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22','X','Y'].index(x[0:]))
+		chromLengths2[genome] = {}
+		totalLength = 0
+		for i in range(0, len(chroms), 1):
+			with open(chrom_path + chroms[i] + ".txt", "rb") as f:
+				chromLengths2[genome][chroms[i]] = totalLength
+				chrom_length = len(f.read())
+				totalLength += chrom_length
+		binsDensity = []
+		for i in range(0, totalLength, windowSize):
+			binsDensity.append(i)
+		binsDensity.append(totalLength)
+
+
 	falsePositives = {}
 	falsePositives_1000 = {}
 
+
 	totalMutsIMD = {}
 	totalMuts1000 = {}
 	if not chrom_based:
@@ -1054,8 +1151,14 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 		bins.append(genomeLength)
 		count = 1
 		for sample in samplesSet:
-			falsePositives[sample.split("_")[0]] = []
-			falsePositives_1000[sample.split("_")[0]] = []
+			if correction:
+				# regions = list(imdsDataSample_corrected[sample.split("_")[0]].keys())
+				regions = list(imdsDataSample_corrected[sample.split(".")[0]].keys())
+			zorderPlot=0
+			# falsePositives[sample.split("_")[0]] = []
+			# falsePositives_1000[sample.split("_")[0]] = []
+			falsePositives[sample.split(".")[0]] = []
+			falsePositives_1000[sample.split(".")[0]] = []
 			falsePositivesChrom = []
 			falsePositivesChrom_1000 = []
 			totalMutsIMDChrom = []
@@ -1067,11 +1170,14 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 			panel1 = plt.axes([0.07, 0.09, 0.9, 0.6])
 			panel2 = plt.axes([0.07, 0.75, 0.9, 0.15])
 			if not aggregated and not chrom_based_IMD:
-				imd_line = imdsData[sample.split("_")[0]]
+				try:
+					imd_line = imdsData[sample.split("_")[0]]
+				except:
+					imd_line = imdsData[sample]
 				panel1.axhline(imd_line, color='red')
-				panel1.axhline(1000, color='red')
-			if chrom_based_IMD:
-				panel1.axhline(1000, color='red')					
+				# panel1.axhline(1000, color='red')
+			# if chrom_based_IMD:
+			# 	panel1.axhline(1000, color='red')					
 
 			ymax = 0
 			classColor = 0
@@ -1083,7 +1189,7 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 				if sample not in samples[subclass]:
 					classColor += 1
 					continue
-				for chrom in chroms:				
+				for chrom in chroms:	
 					if firstClass:
 						centromere_start = centromeres[genome][chrom][0]
 						centromere_end = centromeres[genome][chrom][1]
@@ -1095,43 +1201,67 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 						panel1.add_patch(plt.Rectangle((centromere_startProp, 0.06), centromereLength, .01, facecolor="red", clip_on=False, transform=plt.gcf().transFigure)) 				
 						if chrom_based_IMD:
 							try:
-								imd_line = imdsData[sample.split("_")[0]][chrom]
+								imd_line = imdsData[sample.split(".")[0]][chrom]
+								# imd_line = imdsData[sample.split("_")[0]][chrom]
 							except:
-								imd_line = imdsDataSample[sample.split("_")[0]]
+								imd_line = imdsDataSample[sample.split(".")[0]]
+								# imd_line = imdsDataSample[sample.split("_")[0]]
 							panel1.plot([chrom_start, chrom_start+chromLengths[genome][chrom]],[imd_line,imd_line], color='red')
 
-					if chrom in allMutations[subclass].loc[sample].index:
-						pos = allMutations[subclass].loc[sample, chrom]
+					if str(chrom) in [str(x) for x in allMutations[subclass].loc[sample].index]:
+						try:
+							pos = allMutations[subclass].loc[sample, str(chrom)]
+						except:
+							pos = allMutations[subclass].loc[sample, int(chrom)]
 
 						try:
 							starts = list(pos.loc[(sample, chrom), 'start'])
+							imds_recorded = list(pos.loc[(sample, chrom), 'IMD'])
 							imds = [y-x for x,y in zip(starts, starts[1:])]
 							minIMDs = [min(x,y) for x,y in zip([float('inf')] + imds, imds + [float('inf')])]
 							if not aggregated:
 								if subclass == "Class III":
 									if chrom_based_IMD:
 										try:
-											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom]]
+											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split(".")[0]][chrom] or y < 10000]
+											# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom] or y < 10000]
 										except:
-											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]]]
+											plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split(".")[0]] or y < 10000]
+											# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]] or y < 10000]
 									else:
-										plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]]]
+										plotIMDs = [[x,y] for x,y in zip(starts, imds_recorded)]# if y <= imdsData[sample.split("_")[0]] or y < 10000]
+										# plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]] or y < 10000]
 								else:
 									plotIMDs = [[x,y] for x,y in zip(starts, minIMDs)]
 								plotX, plotY = zip(*plotIMDs)
 								newMax = max(plotY)
 								if chrom_based_IMD:
-									falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split("_")[0]][chrom]]))
+									# falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split("_")[0]][chrom]]))
+									falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split(".")[0]][chrom]]))
 									falsePositivesChrom_1000.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=1000]))
 								else:
-									falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split("_")[0]]]))
+									falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split(".")[0]]]))
+									# falsePositivesChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=imdsData[sample.split("_")[0]]]))
 									falsePositivesChrom_1000.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <=1000]))
 
-								try:
-									totalMutsIMDChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom]]))
-								except:
-									totalMutsIMDChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]]]))
+								# try:
+								# 	totalMutsIMDChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= imdsData[sample.split("_")[0]][chrom]]))
+								# except:
+								# 	totalMutsIMDChrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= imdsDataSample[sample.split("_")[0]]]))
 								totalMuts1000Chrom.append(len([[x,y] for x,y in zip(starts, minIMDs) if y <= 1000]))
+								if correction and subclass != "Non-clust":
+									correctionMuts = []
+									for imd, start in zip(minIMDs, starts):
+									# 	position = int(start) + chromLengths2[genome][chrom]
+									# 	try:
+									# 		bisectRegion = regions[bisect.bisect_left(regions, position)]
+									# 	except:
+									# 		continue
+									# 	if bisectRegion - position < windowSize:
+									# 		imdCorrected = imdsDataSample_corrected[sample.split("_")[0]][bisectRegion]
+										# if imd < imdCorrected and imd > imdsDataSample[sample.split("_")[0]]:
+										correctionMuts.append(1)
+									totalMutsIMDChrom.append(len(correctionMuts))
 							else:
 								plotIMDs = [[x,y] for x,y in zip(starts, minIMDs) if y <= 10000]
 								plotX, plotY = zip(*plotIMDs)
@@ -1149,16 +1279,21 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 							ymax = newMax
 						plotX = [x + chrom_start for x in plotX]
 						densityMuts += plotX
-						panel1.scatter(plotX, plotY, color=colors[subclass], clip_on=False, s=12, label=subclass)
+						if subclass == "Non-clust":
+							zorderPlot = -10
+						panel1.scatter(plotX, plotY, color=colors[subclass], clip_on=False, s=12, label=subclass, zorder=zorderPlot)
+						
 					chrom_start += chromLengths[genome][chrom] + 10000
 				classColor += 1
 				firstClass = False
+				zorderPlot -= 1
 			totalMutsIMD[sample] = sum(totalMutsIMDChrom)
 			totalMuts1000[sample] = sum(totalMuts1000Chrom)
 
-			falsePositives[sample.split("_")[0]].append(sum(falsePositivesChrom))
-			falsePositives_1000[sample.split("_")[0]].append(sum(falsePositivesChrom_1000))
-
+			# falsePositives[sample.split("_")[0]].append(sum(falsePositivesChrom))
+			# falsePositives_1000[sample.split("_")[0]].append(sum(falsePositivesChrom_1000))
+			falsePositives[sample.split(".")[0]].append(sum(falsePositivesChrom))
+			falsePositives_1000[sample.split(".")[0]].append(sum(falsePositivesChrom_1000))
 			hist, bins = np.histogram(densityMuts, bins=bins, density=True)
 			panel2.fill_between(bins[:-1], hist, zorder=10)
 			if ymax > 10**5:
@@ -1189,9 +1324,9 @@ def rainfall (chrom_based_IMD, project, project_path, chrom_path, windowSize = 1
 			pp.savefig(plot1)
 			plt.close()
 
-		with open(projectPath_parent + "output/simulations/data/originalCounts" + path_suffix + ".pickle", "wb") as f:
-			pickle.dump(totalMutsIMD, f)
-		with open(projectPath_parent + "output/simulations/data/originalCounts1000" + path_suffix + ".pickle", "wb") as f:
+		# with open(projectPath_parent + "output/simulations/data/originalCounts" + path_suffix2 + ".pickle", "wb") as f:
+		# 	pickle.dump(totalMutsIMD, f)
+		with open(projectPath_parent + "output/simulations/data/originalCounts1000" + path_suffix2 + ".pickle", "wb") as f:
 			pickle.dump(totalMuts1000, f)
 	else:
 		for chrom in chroms:
